from functools import wraps
from flask import Flask, url_for, render_template, request
app = Flask(__name__)

from pymongo import Connection
db = Connection(port=int(29000)).research

def ellcurves_list(f):
    from utils import LazyMongoDBPagination
    @wraps(f)
    def wrapper(**kwargs):
        kwds = f(**kwargs)
        pagination = LazyMongoDBPagination(query=kwds.pop('curves'),
                                       per_page=50,
                                       page=request.args.get('page', 1),
                                       endpoint=f.__name__,
                                       endpoint_params=kwargs)

        return render_template(f.__name__ + '.html',
                               pagination=pagination, **kwds)
    return wrapper

@app.route('/ellcurves/conductor/<query>')
def ellcurves(query):
    import re
    values = map(int, re.findall(r'(\d+)', query))
    if len(values) == 2:
        return ellcurves_conductor_range(*values)
    elif len(values) == 1:
        return ellcurves_of_conductor(*values)
    else:
        return render_template('invalid_query.html', query=query)

@app.route('/ellcurves/conductor_range/<int:start>/<int:stop>/')
@ellcurves_list
def ellcurves_conductor_range(start, stop):
    curves = db.ellcurves.find({'level':{'$gte':start, '$lte':stop}})
    return locals()

@app.route('/ellcurves/conductor/<int:conductor>/')
@ellcurves_list
def ellcurves_of_conductor(conductor):
    curves = db.ellcurves.find({'level':conductor})
    return locals()

@app.route('/ellcurves/rank/<int:rank>/')
@ellcurves_list
def ellcurves_of_rank(rank):
    curves = db.ellcurves.find({'r':rank}).sort('level')
    return locals()

@app.route('/ellcurves/torsion/<int:torsion>/')
@ellcurves_list
def ellcurves_of_torsion(torsion):
    curves = db.ellcurves.find({'T':torsion}).sort('level')
    return locals()

@app.route('/ellcurve')
def ellcurve():
    a = request.args
    level = int(a.get('level','11'))
    iso_class = a.get('iso_class', 'a')
    number = int(a.get('number', 1))
    cursor = db.ellcurves.find({'level':level, 'iso_class':iso_class, 'number':number})
    return render_template('ellcurve.html', count=cursor.count(True),
                           curve=cursor.next())    

@app.route('/')
def index():
    return render_template('index.html')

@app.route('/slow')
def slow():
    import time
    time.sleep(10)
    return "Finished"

if __name__ == '__main__':
    app.run(debug=True,host='0.0.0.0', port=8765)


